//********************************************************************
//*** ResourcePool/SQL/SQLTransaction.t.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLTransaction.t.cc,v 1.6 2009-05-02 22:17:15 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

using namespace fatalmind;


void test_init();

void init_SQLTransaction(RP& p)
{
	test_init();
	init(p);	
	RP::Commands::SQLExecute init(p, "insert into hirsch (id, ukn) values (?, ?)");
	for (int i = 0; i < 300; ++i) {
		init.bind(0, i);
		init.bind(1, "hirschig");
		p.execute(init);
	}
}

void SQLTransaction_init_me(RP& p) {
	RP::Commands::SQLExecute init(p, "update hirsch set ukn=?");
	init.bind(0, "hohoho ' kasdfjs \" sdf");
	p.execute(init);
}

void trans_test_ok(RP& p, int id, bool cache = false) {
	RP::PoolType::resource_t& h = p.get();
	RP::Commands::SQLBatch bc(p);
	
	RP::Commands::SQLExecute cmd(p, "update hirsch set ukn=? where id =?");
	cmd.bind(1, id*2);
	cmd.bind(0, "hiiiirsch'");
	bc.add(cmd);
	cmd.bind(1, id*2+1);
	bc.add(cmd);

	RP::Commands::Transaction t(p, bc);
	p.execute(t);

	RP::Commands::SQLSelectRow sel(p, "select ukn from hirsch where id = ?");
	sel.bind(0, id*2);
	std::string ukn;
	sel.bindout(0, ukn);
	sel.execute(h);
	assertEqual(ukn, "hiiiirsch'");
	p.free(h);
}

void trans_test_rec_ok(RP& p, int id, bool fail = false) {
	RP::Commands::SQLBatch bc1(p);
	RP::Commands::SQLBatch bc2(p);

	
	RP::Commands::SQLExecute cmd(p, "update hirsch set ukn=? where id =?");
	cmd.bind(1, id*3);
	cmd.bind(0, "hiiiirsch'");
	bc1.add(cmd);
	cmd.bind(1, id*3+1);
	bc1.add(cmd);

	cmd.bind(fail?2:1, id*3+2);
	bc2.add(cmd);
	bc2.add(bc1);

	RP::Commands::Transaction t(p, bc2);

	try {
		p.execute(t);
		if (fail) {
			assertNotReach();	
		}
	} catch (SQL::SQLException) {
		if (!fail) {
			assertNotReach();	
		}
	}
}

void SQLTransaction_rec(RP& p)
{
	RP::PoolType::resource_t& h = p.get();
	SQLTransaction_init_me(p);
	trans_test_rec_ok(p, 0);

	RP::Commands::SQLSelectRow sel(p, "select ukn from hirsch where id = ?");
	sel.bind(0, 0);
	std::string ukn;
	sel.bindout(0, ukn);
	sel.execute(h);
	assertEqual(ukn, "hiiiirsch'");
	p.free(h);
}

void SQLTransaction_rec_fail(RP& p)
{
	RP::PoolType::resource_t& h = p.get();
	SQLTransaction_init_me(p);
	trans_test_rec_ok(p, 0, true);

	RP::Commands::SQLSelectRow sel(p, "select ukn from hirsch where id = ?");
	sel.bind(0, 1);
	std::string ukn;
	sel.bindout(0, ukn);
	sel.execute(h);
	assertEqual(ukn, "hohoho ' kasdfjs \" sdf");
	p.free(h);
}

void trans_test_fail(RP& p, int id, bool cached = false) {
	RP::Commands::SQLBatch bc(p);
	
	RP::Commands::SQLExecute cmd(p, "update hirsch set ukn=? where id =?");
	cmd.bind(0, "gnu");
	cmd.bind(1, id*2);
	bc.add(cmd);

	std::string chk;
	RP::Commands::SQLSelectRow chksel(p, "select ukn from hirsch where id = ?");
	chksel.bind(0, id*2);
	chksel.bindout(0, chk);
	bc.add(chksel);

	// expected to fail because paramter 0 is not bound
	RP::Commands::SQLExecute cmd2(p, "update hirsch set ukn=? where id =?");
	cmd2.bind(1, id*2+1);
	bc.add(cmd2);
	
	RP::Commands::Transaction t(p, bc);

	try {
		p.execute(t);
		assertNotReach();
	} catch (SQL::SQLException e) {
	}
	RP::Commands::SQLSelectRow sel(p, "select ukn from hirsch where id = ?");
	sel.bind(0, id*2);
	std::string ukn;
	sel.bindout(0, ukn);
	p.execute(sel);
	assertEqual(ukn, "hohoho ' kasdfjs \" sdf");
	assertEqual(chk, "gnu");
}

void trans_test_fail_rec(RP& p, int id, bool cache = false) {
	RP::Commands::SQLBatch bc(p);
	RP::Commands::SQLBatch bcsub(p);
	
	RP::Commands::SQLExecute cmd(p, "update hirsch set ukn=? where id =?");
	cmd.bind(0, "gnu2");
	cmd.bind(1, id*2);
	bcsub.add(cmd);
	bc.add(bcsub);

	std::string chk;
	RP::Commands::SQLSelectRow chksel(p, "select ukn from hirsch where id = ?");
	chksel.bind(0, id*2);
	chksel.bindout(0, chk);
	bc.add(chksel);

	RP::Commands::SQLExecute cmd2(p, "update hirsch set ukn=? where id =?");
	cmd2.bind(1, id*2+1);
	bc.add(cmd2);

	RP::Commands::Transaction t(p, bc);
	
	try {
		p.execute(t);
		assertNotReach();
	} catch (SQL::SQLException e) {
	}
	RP::Commands::SQLSelectRow sel(p, "select ukn from hirsch where id = ?");
	sel.bind(0, id*2);
	std::string ukn;
	sel.bindout(0, ukn);
	p.execute(sel);
	assertEqual(ukn, "hohoho ' kasdfjs \" sdf");
	assertEqual(chk, "gnu2");
}

