//********************************************************************
//*** ResourcePool/mysql/mysql_bind.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: mysql_bind.cc,v 1.3 2009-03-22 14:34:20 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <stdlib.h>
#include <mysql/mysql.h>
#include "mysql_bind.hh"
#include "MysqlException.hh"

namespace fatalmind {
namespace mysql {

mysql_bind::mysql_bind(MYSQL& dbh, const SQL::SQLStatement& _sql, const std::vector<gc_ptr<SQLType> >& args) 
: _os()
, _dbh(dbh)
{
	unsigned int p = _sql.getParamCount();
	if (p != args.size()) {
		throw NFOMysqlException("Provided parameter count does not match the number of placeholders in the statatemet", 98, _sql);
	}
	
	for (unsigned int i = 0; i < p; ++i) {
		_os << _sql.getSQLFragment(i);
		appendValue(i, args[i], _sql);
	}
	_os << (_sql.getSQLFragment(p)); // the last fragment
}

void mysql_bind::appendValue(int i, const gc_ptr<SQLType>& v, const SQL::SQLStatement& _sql) {
	if (v.get() == 0) {
		std::ostringstream os;
		os << "Bind parameter " << i << " not specified";
		throw NFOMysqlException(os.str(), 98, _sql);
	}

	const fatalmind::mysql::SQLType& val = *v;
	if (val.isnull()) {
		_os << "null";
	} else if (val.needsquote()) {
		// TODO, to string operation, to be optimized!
		// in most of the cases it is a string!
		std::ostringstream os;
		os << val;
		string strval(os.str());

		// alloca?, TODO, would also save the try/catch
		char* buf = (char*)malloc(strval.length()*2+1);	
		try {
			unsigned long blen = mysql_real_escape_string(&_dbh, buf, strval.data(), strval.length());
			string strbuf(buf, blen); // required because of \0 awareness :(
			_os << '\'' << strbuf << '\'';
			free(buf);
		} catch (...) {
			free(buf);
		}

	} else {
		_os << val;
	}
}

} // namespace mysql
} // namespace fatalmind

