//********************************************************************
//*** ResourcePool/oracle/SQLExecute.t.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLExecute.t.cc,v 1.28 2009-05-02 22:17:15 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#define  FATALMIND_TEST_SHOW_PROTECTED

#include "ResourcePool/oracle/ResourcePool.hh"
#include "t/main.hh"

#include "testconfig.cc"

#include "../SQL/SQLExecute.t.cc"

std::auto_ptr<RP> rp;

#ifndef RESOURCEPOOL_ORACLE_ORACLETYPE_HH
#include "oracleType.hh"
#endif
using fatalmind::oracle::oracleType;
using fatalmind::oracle::oracleTypeImpl;

void test_init()
{
    if (rp.get() == 0) {
        ResourcePoolOptions opt;
        opt.setMaxExecTry(1);
        rp.reset(new RP(getFactory(), opt));
    }
}

void INIT_oracle_SQLExecute() {
    rp.reset(new RP(getFactory()));
    test_SQLExecute_init(*rp);
}


void TEST_oracle_SQLExecute_insert() {
	test_SQLExecute_one(*rp);
}

void TEST_oracle_SQLExecute_insert_two() {
	test_SQLExecute_two(*rp);
}

void TEST_oracle_SQLExecute_insert_three() {
	test_SQLExecute_three(*rp);
}


void TEST_PERF_oracle_SQLExecute_massbind() {
	test_perf_SQLExecute_massbind(*rp);
}

void TEST_PERF_oracle_SQLExecute_insert() {
	rp->downsize();
	test_perf_SQLExecute_insert(*rp);
}

void TEST_PERF_oracle_SQLExecute_insert_outer() {
	test_perf_SQLExecute_insert_outer(*rp);
}

void TEST_PERF_oracle_SQLExecute_update_outer() {
	test_perf_SQLExecute_update_outer(*rp);
}

void TEST_oracle_SQLExecute_insert_fail() {
	test_SQLExecute_fail(*rp);
}

void TEST_oracle_SQLExecute_output() {
	test_SQLExecute_output(*rp);
}

void init2(RP& p) {
	drop(p);
	try {
	        RP::Commands::SQLExecute sql(
			"create table hirsch (id numeric not null primary key, ukn varchar(255))"
		);
		p.execute(sql);
	} catch (fatalmind::SQL::SQLException& e) {
		// TODO, filter ORA-00955
		// happens only if sthcache is active, maye DDL's should not be kept open?
	}
        RP::Commands::SQLExecute sql2(
                "alter session set sql_trace=true"
        );
        p.execute(sql2);
}

void TEST_oracle_SQLExecute_batched() {
	init2(*rp);
	std::string insertsql = "insert into hirsch (id, ukn) values (?, ?)";
	RP::Commands::SQLExecute param(*rp, insertsql);
	RP::Commands::SQLExecute param2(*rp, insertsql);
	RP::Commands::SQLExecute param3(*rp, insertsql);
	RP::Commands::SQLExecute cmd(*rp, insertsql);
	cmd.bind(0, 900);
	cmd.bind(1, "testme1");

	param.bind(0, 901);
	param.bind(1, "testme b");
	param2.bind(0, 902);
	param2.bind(1, "testme c");
	param3.bind(0, 903);
	param3.bind(1, "testme d");
	
	assertTrue(cmd.addBatch(param));
	assertTrue(cmd.addBatch(param2));
	assertTrue(cmd.addBatch(param3));

	rp->execute(cmd);

	RP::Commands::SQLSelectRow sel(*rp, "select id from hirsch where id=?");
	int out = 0;
	sel.bindout(0, out);
	sel.bind(0, 901);
	rp->execute(sel);

	// update it
	assertEqual(out, 901);
	std::string updatesql = "update hirsch set ukn = ? where id <= ?";
	RP::Commands::SQLExecute up(*rp, updatesql);
	up.bind(0, "updated");
	up.bind(1, 1000); // hits all 4 records

	RP::Commands::SQLExecute param4(*rp, updatesql);
	param4.bind(0, "fits");
	param4.bind(1, 900); // hits one record

	RP::Commands::SQLExecute param5(*rp, updatesql);
	param5.bind(0, "also too");
	param5.bind(1, 902);

	assertTrue(up.addBatch(param4));
	assertTrue(up.addBatch(param5));
	rp->execute(up);

	std::string result;
	RP::Commands::SQLSelectRow sel2(*rp, "select ukn from hirsch where id = ?");
	sel2.bind(0, 903);
	sel2.bindout(0, result);
	rp->execute(sel2);
	assertEqual(result, "updated");

	// check re-execution of up
	RP::Commands::SQLExecute up2(*rp, "update hirsch set ukn = 'changed' where id = 903");
	rp->execute(up2);
	rp->execute(sel2);
	assertEqual(result, "changed");

	rp->execute(up);
	rp->execute(sel2);
	assertEqual(result, "updated");
}

void TEST_oracle_SQLExecute_batched_fail() {
	init2(*rp);
	std::string insertsql = "insert into hirsch (id, ukn) values (?, ?)";
	std::string insertsql2 = "insert into HIRSCH    (id, ukn) values (?, ?)";
	RP::Commands::SQLExecute param(*rp, insertsql2);
	RP::Commands::SQLExecute cmd(*rp, insertsql);
	cmd.bind(0, 900);
	cmd.bind(1, "testme1");
	param.bind(0, 901);
	param.bind(1, "testme b");

	// different SQL
	assertFalse(cmd.addBatch(param));

	RP::Commands::SQLExecute param2(*rp, insertsql);
	try {
		cmd.addBatch(param2);
		// number of parameters don't match
		assertNotReach();
	} catch (...) {
	}

	param2.bind(0, 100);	
	param2.bind(1, 101); // different type
	assertFalse(cmd.addBatch(param2));
	assertEqual(cmd.batchSize(), 1); // only one statement
	
	rp->execute(cmd); // execution is possible

	
	RP::Commands::SQLSelectRow sel(*rp, "select count(*) from hirsch");
	int count = 0 ;
	sel.bindout(0, count);
	rp->execute(sel);
	assertEqual(count, 1); // only one statement executed

	RP::Commands::SQLExecute exceptioncmd(*rp, insertsql);
	exceptioncmd.bind(0, 2000);
	exceptioncmd.bind(1, "works");

	RP::Commands::SQLExecute exceptioncmd2(*rp, insertsql);
	// unique constraint violaion
	exceptioncmd2.bind(0, 2000);
	exceptioncmd2.bind(1, "blah");
	assertTrue(exceptioncmd.addBatch(exceptioncmd2));
	// TODO: define TX handling in case of OCI_BATCH_ERRORS and failure!
	RP::Commands::Transaction t(exceptioncmd);
	for (int i = 0; i < 500; ++i) {
		try {
			rp->execute(t);
			assertNotReach();
		} catch (fatalmind::Exception e) {
		}
	}

}

// a complex failure and recover scenario
void TEST_oracle_SQLExecute_batched_fail_mixed() {
	init2(*rp);
	std::string insertsql = "insert into hirsch (id, ukn) values (?, ?)";
	RP::Commands::SQLExecute cmd(*rp, insertsql);
	cmd.bind(0, 1);
	cmd.bind(1, "testme 1");

	RP::Commands::SQLExecute blah(*rp, insertsql);
	blah.bind(0, 2);
	blah.bind(1, "works 2");
	assertTrue(cmd.addBatch(blah));

	//RP::Commands::SQLExecute blah(*rp, insertsql);
	blah.bind(0, 2);
	blah.bind(1, "doesn't work");
	assertTrue(cmd.addBatch(blah));

	//RP::Commands::SQLExecute blah(*rp, insertsql);
	blah.bind(0, 3);
	blah.bind(1, "works 3");
	assertTrue(cmd.addBatch(blah));

	//RP::Commands::SQLExecute blah(*rp, insertsql);
	blah.bind(0, 4);
	blah.bind(1, "works 4");
	assertTrue(cmd.addBatch(blah));
	
	//RP::Commands::SQLExecute blah(*rp, insertsql);
	blah.bind(0, 4);
	blah.bind(1, "doesn't work");
	assertTrue(cmd.addBatch(blah));
	
	//RP::Commands::SQLExecute blah(*rp, insertsql);
	blah.bind(0, 5);
	blah.bind(1, "works 5");
	assertTrue(cmd.addBatch(blah));
	
	// till here we have build a batch of 7 commands:
	// offset 0 = 1 - testme 1
	// offset 1 = 2 - works 2
	// offset 2 = 2 - doesn't work
	// offset 3 = 3 - works 3
	// offset 4 = 4 - works 4
	// offset 5 = 4 - doesn't work
	// offset 6 = 5 - works 5

	// execute it according to the proposed structure
	try {
		rp->execute(cmd);
		assertNotReach();
	} catch (fatalmind::SQL::SQLException e) {
		// should return the first exception for the insert 2 - doesn't work
		assertEqual(cmd.getBatchExceptionOffset(), 2);
	}

	// get the second exception
	try {
		cmd.throwBatchException();
		assertNotReach();
	} catch (fatalmind::SQL::SQLException e) {
		// should return the second exception for the 4 - doesn't work
		assertEqual(cmd.getBatchExceptionOffset(), 5);
	}

	// proof that there is no more exception
	cmd.throwBatchException();
	assertEqual(cmd.getBatchExceptionOffset(), -2);
}

void CLEANUP_oracle_SQLExecute() {
	drop(*rp);
}
