//********************************************************************
//*** util/Clone.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: Clone.hh,v 1.10 2009-03-02 14:16:47 mws Exp $
//********************************************************************

#ifndef UTIL_CLONE_HH
#define UTIL_CLONE_HH

#include <typeinfo>

#include "Exceptions/Exception.hh"

namespace fatalmind {

class Clone;

class CloneException: public Exception {
	public:
		CloneException(const Clone* const t1);
	private:
		static std::string getCloneErrorMessage(const Clone* const t1);
};

// according to C++ Coding Standards, Herb Sutter, Andrei Alexandrescu 2005
class Clone {
	public:
		Clone* clone() const {
			return clone<Clone>();
		}

		template<class Derived>
		Derived* clone() const {
			Clone* c = DoClone();
			if (typeid(*c) != typeid(*this)) {
				throw CloneException(this);
			}
			Derived& d = dynamic_cast<Derived&>(*c);
			return &d;
		}
	protected:
		virtual Clone* DoClone() const = 0;
		virtual ~Clone() {
		}
};

}

#endif
